#!/bin/sh

build_module () {
	echo =====================================
	echo build module $1 \($2\)
	echo =====================================
	echo ------------
	echo build source
        echo ------------
	cd src/modules
	module_version=`perl -MExtUtils::MakeMaker -le "print MM->parse_version(shift)" "openca-${2}/${1}.pm"`
	cp -r openca-${2} OpenCA-${1}-${module_version}
	tar cpf OpenCA-${1}-${module_version}.tar --exclude "CVS" OpenCA-${1}-${module_version}
	gzip OpenCA-${1}-${module_version}.tar
	mv OpenCA-${1}-${module_version}.tar.gz ${SOURCES}
	rm -rf OpenCA-${1}-${module_version}
	echo ---------
	echo build RPM
        echo ---------
	rpm --clean -ba openca-${2}/specs/${DISTRIBUTION}.spec
	echo -------
	echo cleanup
	echo -------
	rm -f ${SOURCES}/OpenCA-${1}-${module_version}.tar.gz
	cd ../..
	## all RPMs are placed in the root of the source
	## some distributions like SuSE use perl-Module-Name
	mv ${RPMS}/*/*OpenCA-${1}-*.rpm ..
	mv ${SRPMS}/*OpenCA-${1}-*.rpm ..
}

build_c_module () {
	echo =====================================
	echo build C-module $1 \($2\)
	echo =====================================
	echo ------------
	echo build source
        echo ------------
	cd src/
	module_version=`cat ${2}/VERSION`
	cp -r ${2} OpenCA-${1}-${module_version}
	tar cpf OpenCA-${1}-${module_version}.tar --exclude "CVS" OpenCA-${1}-${module_version}
	gzip OpenCA-${1}-${module_version}.tar
	mv OpenCA-${1}-${module_version}.tar.gz ${SOURCES}
	rm -rf OpenCA-${1}-${module_version}
	echo ---------
	echo build RPM
        echo ---------
	rpm --clean -ba ${2}/specs/${DISTRIBUTION}.spec
	echo -------
	echo cleanup
	echo -------
	rm -f ${SOURCES}/OpenCA-${1}-${module_version}.tar.gz
	cd ..
	## all RPMs are placed in the root of the source
	## some distributions like SuSE use perl-Module-Name
	mv ${RPMS}/*/*OpenCA-${1}-*.rpm ..
	mv ${SRPMS}/*OpenCA-${1}-*.rpm ..
}

build_interfaces () {
	echo ============================================
	echo build interfaces and common: $1 \($2\)
	echo ============================================

	echo ------------
	echo build source
        echo ------------

	pkg=OpenCA
	arc="${ver}.tar.gz"

	pkg_module=${pkg}-${1}
	arc_module="${pkg_module}-${arc}"

	spec_module=openca-${2}
	spec_module="contrib/rpm/${spec_module}.spec"

	export PACKAGE=${1}
	make source
	cp "${arc_module}" "${SOURCES}"
	rpm -ba ${spec_module}
	mv ${RPMS}/*/${pkg_module}*.rpm ..
	mv ${SRPMS}/${pkg_module}*.rpm ..
	rm -f ${SOURCES}/${arc_module}
}

echo ===========================
echo starting to create the RPMs
echo ===========================

ver=0.9.1
export ver

echo ==============
echo Version: $ver
echo ==============

# default is redhat
DISTRIBUTION=redhat
RPM_BASE=/usr/src/redhat
SOURCES=${RPM_BASE}/SOURCES
SRPMS=${RPM_BASE}/SRPMS
RPMS=${RPM_BASE}/RPMS
OPENCA_CORE=/usr/local/openca.${ver}
CGIBIN=/usr/local/openca.${ver}/httpd/cgi-bin
HTDOCS=/usr/local/openca.${ver}/httpd/htdocs

if [ "$2" = "suse" ] ; then
	DISTRIBUTION=suse
	RPM_BASE=/usr/src/packages
	SOURCES=${RPM_BASE}/SOURCES
	SRPMS=${RPM_BASE}/SRPMS
	RPMS=${RPM_BASE}/RPMS
	OPENCA_CORE=/usr/local/openca.${ver}
	CGIBIN=/usr/local/httpd/cgi-bin
	HTDOCS=/usr/local/httpd/htdocs
fi

export DISTRIBUTION OPENCA_CORE CGIBIN HTDOCS

echo ===========================
echo build all for $DISTRIBUTION
echo ===========================

if [ "$1" = "all" ]; then
	build_c_modules=yes
	build_common=yes
	build_interfaces=yes
	build_modules=yes
else
	build_c_modules=no
	build_common=no
	build_interfaces=no
	build_modules=no
	build_${1}=yes
fi

if [ "${build_common}" = "yes" ] ; then
	build_interfaces "common" "common"
fi

if [ "${build_interfaces}" = "yes" ] ; then
	build_interfaces "CA" "ca"
	build_interfaces "CA-Node" "ca-node"
	build_interfaces "LDAP" "ldap"
	build_interfaces "Public" "pub"
	build_interfaces "RA" "ra"
	build_interfaces "RA-Node" "ra-node"
fi

if [ "${build_c_modules}" = "yes" ] ; then
	build_c_module "OCSPD" "ocspd"
	build_c_module "SV" "openca-sv"
	## build_module "SCEP" "scep"
fi

if [ "${build_modules}" = "yes" ] ; then
	build_module "Configuration" "configuration"
	build_module "CRL" "crl"
	build_module "DB" "db"
	build_module "DBI" "dbi"
	build_module "OpenSSL" "openssl"
	build_module "PKCS7" "pkcs7"
	build_module "REQ" "req"
	build_module "Tools" "tools"
	build_module "TRIStateCGI" "tristatecgi"
	build_module "X509" "x509"
fi

exit 0;
