/* src/net/server.c 
 * ============================================================
 * OCSP Responder
 * (c) 2001 by Massimiliano Pala
 * OpenCA released software
 * ============================================================
 */
 
#include "general.h"

#include <resolv.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <openssl/err.h>
#include <openssl/ssl.h>

#include <openssl/conf.h>
#include <openssl/txt_db.h>

#include "support.h"
#include "ocsp_response.h"
#include "http_request.h"
#include "server.h"

/* External General Variables */
extern int debug;
extern int verbose;

int start_server( char *bind_s, char *port_s, int queue, long max_size, CONF *conf,
	          EVP_PKEY *pkey, X509 *ocspd_cert, X509 *cacert, TXT_DB *db ) {

	struct sockaddr_in sin;
	int serverSocket;
	int tmp_socket;
	int address_size;

	serverSocket = socket(AF_INET, SOCK_STREAM, 0);
	bzero( &sin, sizeof(sin));

	sin.sin_family 	= AF_INET;
	sin.sin_addr.s_addr 	= INADDR_ANY;
	sin.sin_port 		= htons ( atoi(port_s) );
	sin.sin_addr.s_addr 	= htonl(INADDR_ANY);

	bind( serverSocket, (struct sockaddr *)&sin,
			sizeof(sin));


	/* No max size under 1024 */
	if( max_size < 1024 ) max_size = 1024;
	
	if (verbose)
		syslog(LOG_INFO, "successfully binded to %d", atoi(port_s) );

	while ( 1 ) {
		handleConnection( serverSocket );
	}

	return(0);
}


int handleConnection( int serverSocket ) {

	struct sockaddr_in pin;
	int address_size;

	int pid;
	int listen4_fd;
	int conn_fd;

	listen4_fd = listen( serverSocket, 20 );

	conn_fd = accept ( serverSocket, (struct sockaddr *)&pin,
		&address_size );	

	pid = fork();
	if( pid < 0 ) {
		/* Error */
		syslog(LOG_ERR,"cannot spawn child");
	} else if ( pid == 0 ) {
		/* Client: handle the connection */
		close( listen4_fd );
		handle_request( conn_fd );
		exit(0);
	}
	close( conn_fd );
}

void handle_request( int conn_fd ) {

	HTTP_REQUEST *pReq = NULL;

	char buf[10000];
	char *req = NULL;
	int req_len = 0;

	int max_size = 10000;
	int r;

	for( ;; ) {
		r = read( conn_fd, buf, max_size );

		if( r <= 0 ) break;

		if( req == NULL ) {
			req = (char *) malloc ( r + 1 );
			memcpy( req, buf, r );
			req[r] = 0;
			
			req_len = r;
		} else {
			char *tmp_buf;

			tmp_buf = (char *) malloc ( req_len + r + 1 );
			memcpy( tmp_buf, req, req_len );

			free(req);

			memcpy( &tmp_buf[req_len + 1], buf, r );
			req = tmp_buf;
			req_len += r;

			req[req_len] = 0;
		}

		if ( ((char *)strstr(req,"\r\n\r\n" ) != NULL) ||
			((char *)strstr( req, "\n\n" ) != NULL) )
			break;
	}

	if((ocsp_req=(OCSP_REQUEST *) OPENSSL_malloc (sizeof(OCSP_REQUEST)))
									== NULL) {
		return;
	}

	ocsp_req = d2i_OCSP_REQUEST_bio(cbio, NULL);
	if (!ocsp_req) {
		syslog( LOG_ERR, "Error parsing OCSP request\n");
	}

	pReq = parse_http_request( req, req_len );
	req = d2i_OCSP_REQUEST( bio, NULL );

	printf("REQUEST:\n%s\n", req );

	return;

}
